// SerialServer.cpp : main source file for SerialServer.exe
//

#include "stdafx.h"

#include "resource.h"
#include "Trace.h"
#include "messages.h"

// Note: Proxy/Stub Information
//		To build a separate proxy/stub DLL, 
//		run nmake -f SerialServerps.mk in the project directory.
#include "initguid.h"
#include "SerialServer.h"
#include "SerialServer_i.c"

#include "aboutdlg.h"
#include "maindlg.h"

CString  g_csPath = "";
HWND     g_hMainDlg = NULL;
int      g_iTraceLevel = 1;

DWORD  g_dwTraceThreadId = NULL;
HANDLE g_hTraceThreadHandle = NULL;
HANDLE g_hTraceMsg = NULL;
unsigned __stdcall TraceThread(LPVOID pParam);

CServerAppModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
END_OBJECT_MAP()

int Run(LPTSTR /*lpstrCmdLine*/ = NULL, int nCmdShow = SW_SHOWDEFAULT)
{
	CMessageLoop theLoop;
	_Module.AddMessageLoop(&theLoop);

	CMainDlg dlgMain;

	if(dlgMain.Create(NULL) == NULL)
	{
		ATLTRACE(_T("Main dialog creation failed!\n"));
		return 0;
	}

	_Module.Lock();
	
	g_hMainDlg = dlgMain.m_hWnd;

	dlgMain.ShowWindow(nCmdShow);

	int nRet = theLoop.Run();

	_Module.RemoveMessageLoop();
	return nRet;
}

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPTSTR lpstrCmdLine, int nCmdShow)
{
	HRESULT hRes = ::CoInitialize(NULL);
// If you are running on NT 4.0 or higher you can use the following call instead to 
// make the EXE free threaded. This means that calls come in on a random RPC thread.
//	HRESULT hRes = ::CoInitializeEx(NULL, COINIT_MULTITHREADED);
	ATLASSERT(SUCCEEDED(hRes));

	// this resolves ATL window thunking problem when Microsoft Layer for Unicode (MSLU) is used
	::DefWindowProc(NULL, 0, 0, 0L);

	AtlInitCommonControls(ICC_COOL_CLASSES | ICC_BAR_CLASSES);	// add flags to support other controls

	hRes = _Module.Init(ObjectMap, hInstance);
	ATLASSERT(SUCCEEDED(hRes));

	AtlAxWinInit();

	// Parse command line, register or unregister or run the server
	int nRet = 0;
	TCHAR szTokens[] = _T("-/");
	bool bRun = true;
	bool bAutomation = false;

	LPCTSTR lpszToken = _Module.FindOneOf(::GetCommandLine(), szTokens);
	while(lpszToken != NULL)
	{
		if(lstrcmpi(lpszToken, _T("UnregServer")) == 0)
		{
			_Module.UpdateRegistryFromResource(IDR_SERIALSERVER, FALSE);
			nRet = _Module.UnregisterServer(TRUE);
			bRun = false;
			break;
		}
		else if(lstrcmpi(lpszToken, _T("RegServer")) == 0)
		{
			_Module.UpdateRegistryFromResource(IDR_SERIALSERVER, TRUE);
			nRet = _Module.RegisterServer(TRUE);
			bRun = false;
			break;
		}
		else if((lstrcmpi(lpszToken, _T("Automation")) == 0) ||
			(lstrcmpi(lpszToken, _T("Embedding")) == 0))
		{
			bAutomation = true;
			break;
		}
		lpszToken = _Module.FindOneOf(lpszToken, szTokens);
	}

	if(bRun)
	{
		_Module.StartMonitor();
		hRes = _Module.RegisterClassObjects(CLSCTX_LOCAL_SERVER, REGCLS_MULTIPLEUSE | REGCLS_SUSPENDED);
		ATLASSERT(SUCCEEDED(hRes));
		hRes = ::CoResumeClassObjects();
		ATLASSERT(SUCCEEDED(hRes));

		if(bAutomation)
		{
			CMessageLoop theLoop;
			nRet = theLoop.Run();
		}
		else
		{
			nRet = Run(lpstrCmdLine, nCmdShow);
		}

		_Module.RevokeClassObjects();
		::Sleep(_Module.m_dwPause);
	}

	_Module.Term();
	::CoUninitialize();

	return nRet;
}

unsigned __stdcall TraceThread(LPVOID pParam)
{
    HRESULT hr = CoInitialize(NULL);
    if (FAILED(hr))
    {
        ATLTRACE("\n\n\n************ Oops: Telegram delivery didn't work ... *************\n\n\n");
        Beep(200,100);
        Beep(100,100);
        Beep(200,100);
        return hr;
    }

	MSG msg;
	BOOL bRet;
    
	PeekMessage(&msg,(HWND)-1,NULL,NULL,PM_NOREMOVE);

	// Thread damit das Schreiben auf platte nicht so aufhlt.

	CTrace* cTrace = new CTrace;
	cTrace->m_hMainDlg = g_hMainDlg;
	
	DWORD State = GetQueueStatus(QS_ALLINPUT);
	ATLTRACE("TraceThread Es Sind noch %d Nachrichten in der queue\n",LOWORD(State)); 

	SetEvent(g_hTraceMsg);

	while (1)
	{
		//mu so sein das er alle nachrichten bekomm sonst tut das fenstern nicht
		bRet = GetMessage(&msg,NULL,WM_NULL,WM_APPEND);
		if (msg.message == WM_SHUTDOWN)
		{
			break;
		}
		else if (msg.message == WM_TRACEIT)
		{
			cTrace->WriteIt(msg.wParam,msg.lParam);
		}
		else if (msg.message == WM_ALIVE)
		{
			cTrace->Alive(msg.wParam,msg.lParam);
		}
		if (bRet != -1)
		{
			TranslateMessage(&msg); 
			DispatchMessage(&msg); 
		}
	}
	
	State = GetQueueStatus(QS_ALLINPUT);
	ATLTRACE("TraceThread Es Sind noch %d Nachrichten in der queue\n",LOWORD(State)); 

	delete cTrace;
	ATLTRACE("TraceThread stopped\n");
    CoUninitialize();
    _endthreadex(0);
    return 0;
}
